<?php

class PrioritizeHooksAdmin {
	const PAGE_NAME = 'prioritize-hooks';
	const OPTION_GROUP = 'priorities';

	//Add a submenu link to Prioritize options page on the Settings menu.
	static function menuSetup() {
		add_options_page(
			__('Prioritize Hooks', 'prioritize-hooks'), //page title
			__('Prioritize Hooks', 'prioritize-hooks'), //menu heading
			'manage_options', //credentials
			'prioritize-hooks', //menu slug
			array('PrioritizeHooksAdminView', 'displayHooks') //display callback
		);
	}

	static function hooksOverrideSetup() {
		$installedcallbacks = apply_filters( 'prioritize_hooks_admin_filter_callbacks', PrioritizeHooks::installedCallbacksByPackage() );

		//Register hooks option and add a section for each installable
		self::registerOption(PrioritizeHooks::tag);
		foreach($installedcallbacks as $installable=>$callbacks) {
			self::registerOptionSection($installable);
			self::registerOptionFields($installable, $callbacks);
		}
	}

	static function validateField($input) {
		foreach( (array)$input['priorities'] as $tag=>$fnames )
			foreach( (array)$fnames as $fname=>$override )
				if( is_numeric($override) )
					$input['priorities'][$tag][$fname] = (int)$override;
				else
					unset($input['priorities'][$tag][$fname]);

		return $input;
	}

	static function loadResources() {
		global $wp_scripts;

		wp_enqueue_script( 'prioritize-hooks-admin-script', plugins_url('/resources/admin-scripts.js', __FILE__), array('jquery-ui-accordion') );
		$ui = $wp_scripts->query('jquery-ui-core');
		wp_enqueue_style( 'jquery-ui', "http://ajax.googleapis.com/ajax/libs/jqueryui/{$ui->ver}/themes/base/jquery-ui.css" );
	}

	private static function registerOption($option) {
		register_setting(self::OPTION_GROUP, //option group
			$option, //option name
			array(__CLASS__, 'validateField') //sanitation callback
		);
	}

	private static function registerOptionSection($section) {
		add_settings_section(
			PrioritizeHooks::tag . "-$section", //id of section
			$section, //title of section
			array('PrioritizeHooksAdminView', 'displaySection'), //display callback
			self::PAGE_NAME //page to display section
		);
	}

	private static function registerOptionFields($section, $fields) {
		$overrides = PrioritizeHooks::getOverridenCallbacks();
		$defaults = PrioritizeHooks::getDefaultPriorities();

		foreach($fields as $index=>$callback) {
			extract($callback, EXTR_PREFIX_ALL, 'cb');
			add_settings_field(
				PrioritizeHooks::tag . "-priority-$index", //id of field
				"$cb_tag:<br/>$cb_fname", //title of field
				array('PrioritizeHooksAdminView', 'displayOptionField'), //callback for populating field
				self::PAGE_NAME, //page to display field
				PrioritizeHooks::tag . "-$section", //section to display field
				array( //args to pass to callback
					'callback' => $callback,
					'override' => isset($overrides[$cb_tag][$cb_fname]) ? $overrides[$cb_tag][$cb_fname] : false,
					'default' => isset($defaults[$cb_tag][$cb_fname]) ? $defaults[$cb_tag][$cb_fname] : false,
					'index' => $index
				)
			);
		}
	}

}

class PrioritizeHooksAdminView {
	//Display Prioritize Hooks configuration.
	static function displayHooks() {
		if (! current_user_can('manage_options') )
			wp_die( __('You do not have sufficient permissions to access this page.') );

		echo '<div class="wrap">';
		echo '<h2>Prioritize Hooks Settings</h2>';
		echo '<form method="post" action="options.php">';
		settings_fields(PrioritizeHooksAdmin::OPTION_GROUP);
		echo '<div id="prioritize-hooks-form-fields">';
		do_settings_sections(PrioritizeHooksAdmin::PAGE_NAME);
		echo '</div>';
		echo '<p class="submit">';
		echo '<input type="submit" name="submit" class="button-primary" value="' . __('Save Changes') . '" />';
		echo '</p>';
		echo '</form>';
		echo '</div>';
	}

	//Displays this after section heading.
	static function displaySection($section) {
	}

	//Renders the setting fields.
	static function displayOptionField($args) {
		extract($args['callback'], EXTR_PREFIX_ALL, 'cb');

		$fieldname = PrioritizeHooks::tag . "[priorities][$cb_tag][$cb_fname]";
		$id = PrioritizeHooks::tag . "-priority-{$args['index']}";
		false !== $value = $args['override'] or $value = '';
		false !== $default = $args['default'] or $default = $cb_priority;

		echo "<input id='$id' name='$fieldname' type='text' size='3' value='$value'/>";
		printf('<span class="default-value">%s: %d</span>',  __('Default', 'prioritize-hooks'), $default);
	}

}

?>